==> ejc-businessplan-impl-3.0.0-20220703.150729-92-sources.jar: com/ejianc/business/change/excel/ExcelChangePlanprojectController.java <==
package com.ejianc.business.change.excel;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ejianc.business.change.bean.ChangePlanprojectdetailEntity;
import com.ejianc.business.change.service.IChangePlanprojectdetailService;
import com.ejianc.business.change.vo.ChangePlanprojectVO;
import com.ejianc.business.change.vo.ChangePlanprojectdetailVO;
import com.ejianc.business.utils.DateUtils;
import com.ejianc.framework.core.exception.BusinessException;
import com.ejianc.framework.core.kit.mapper.BeanMapper;
import com.ejianc.framework.core.response.CommonResponse;
import com.ejianc.framework.core.util.ExcelExport;
import com.ejianc.framework.core.util.ExcelReader;
import com.ejianc.framework.core.util.FileUtils;
import com.ejianc.framework.core.util.ImportTemplate;
import com.ejianc.support.idworker.util.IdWorker;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.Serializable;
import java.math.BigDecimal;
import java.text.ParseException;
import java.util.*;

/**
 * 项目商务策划立项导入导出
 */
@Controller
@RequestMapping("changePlanprojectExport")
public class ExcelChangePlanprojectController implements Serializable {
    private static final long serialVersionUID = 1L;

    private Logger logger = LoggerFactory.getLogger(this.getClass());
    @Autowired
    private IChangePlanprojectdetailService planprojectdetailService;

    /**
     * 导入模板下载
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/downloadPlanproject")
    @ResponseBody
    public void downloadTotalPrice(HttpServletRequest request, HttpServletResponse response) {
        ImportTemplate.initialize(response);
        ImportTemplate.templetdownload(request, "planproject-import.xlsx", "项目商务策划立项清单导入模板");
    }

    /**
     * excel导入
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/excelImportPlanproject", method = RequestMethod.POST)
    @ResponseBody
    public CommonResponse<Object> excelImportTotalPric(HttpServletRequest request, HttpServletResponse response) throws ParseException {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        List<ChangePlanprojectdetailVO> successList = new ArrayList<>();
        List<ChangePlanprojectdetailVO> errorList = new ArrayList<>();
        boolean isFailed = false;
        MultipartFile mf = null;
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            mf = entity.getValue();
            String originalFileName = mf.getOriginalFilename();
            String extName = null;
            originalFileName = originalFileName.replaceAll("\\/|\\/|\\||:|\\?|\\*|\"|<|>|\\p{Cntrl}", "_");
            originalFileName.replaceAll("00.", "");
            extName = FileUtils.getFileExt(originalFileName, false);
            if (!"xls".equals(extName) && !"xlsx".equals(extName)) {
                isFailed = true;
                break;
            }
        }
        if (isFailed) {
            return CommonResponse.error("文件格式不合法");
        } else {
            List<List<String>> result = ExcelReader.readExcel(mf);
            if (result != null && result.size() > 0) {
                for (int i = 1; i < result.size(); i++) {
                    List<String> datas = result.get(i);
                    ChangePlanprojectdetailVO planprojectdetailVO = new ChangePlanprojectdetailVO();
                    planprojectdetailVO.setId(IdWorker.getId());//id
                    String sort = datas.get(0);
                    if (sort == null || StringUtils.isEmpty(sort)) {
                        throw new BusinessException("序号不能为空，请检查!");
                    }
                    planprojectdetailVO.setSort(StringUtils.isNotBlank(datas.get(0)) ? String.valueOf(datas.get(0)) : null);//id
                    planprojectdetailVO.setCategoryAndName(StringUtils.isNotBlank(datas.get(1)) ? datas.get(1) : null); // 类别及项目名称
                    planprojectdetailVO.setInitiateReason(StringUtils.isNotBlank(datas.get(2)) ? datas.get(2) : null); // 立项理由及依据
                    planprojectdetailVO.setInitiateDate(StringUtils.isNotBlank(datas.get(3)) ? DateUtils.Date(datas.get(3)) : null); // 立项时间
                    planprojectdetailVO.setCostEstimate(StringUtils.isNotBlank(datas.get(4)) ? new BigDecimal(datas.get(4)) : null); // 成本估算(成本分析)(元)
                    planprojectdetailVO.setReportAmount(StringUtils.isNotBlank(datas.get(5)) ? new BigDecimal(datas.get(5)) : null); // 上报金额(元)
                    planprojectdetailVO.setTargetAmount(StringUtils.isNotBlank(datas.get(6)) ? new BigDecimal(datas.get(6)) : null); // 策划目标金额(元)
                    planprojectdetailVO.setCompleteAmount(StringUtils.isNotBlank(datas.get(7)) ? new BigDecimal(datas.get(7)) : null); // 已完成目标金额(元)
                    planprojectdetailVO.setEstimateProfits(StringUtils.isNotBlank(datas.get(8)) ? new BigDecimal(datas.get(8)) : null); // 预计利润(元)
                    planprojectdetailVO.setProposedCompleteTime(StringUtils.isNotBlank(datas.get(9)) ? DateUtils.Date(datas.get(9)) : null); // 拟完成时间节点
                    planprojectdetailVO.setCompleteTime(StringUtils.isNotBlank(datas.get(10)) ? DateUtils.Date(datas.get(10)) : null); // 完成时间
                    planprojectdetailVO.setSupportingDetails(StringUtils.isNotBlank(datas.get(11)) ? datas.get(11) : null); // 证明资料明细
                    planprojectdetailVO.setReportToThismonthProgress(StringUtils.isNotBlank(datas.get(12)) ? new BigDecimal(datas.get(12)) : null); // 报公司截至本月进度(元)
                    planprojectdetailVO.setFirstResponsiblePerson(StringUtils.isNotBlank(datas.get(13)) ? datas.get(13) : null); // 第一责任人
                    planprojectdetailVO.setMainResponsiblePerson(StringUtils.isNotBlank(datas.get(14)) ? datas.get(14) : null); // 主要负责人
                    planprojectdetailVO.setFinalCompleteTime(StringUtils.isNotBlank(datas.get(15)) ? DateUtils.Date(datas.get(15)) : null); // (最后)完成时间
                    planprojectdetailVO.setAttention(StringUtils.isNotBlank(datas.get(16)) ? datas.get(16) : null); // 推荐情况、存在问题、注意事项
                    if(sort==null || sort == ""){
                        planprojectdetailVO.setErrorMsg("序号不能为空");
                        errorList.add(planprojectdetailVO);
                        continue;
                    }else{
                        planprojectdetailVO.setSort(datas.get(0));
                    }
                    if(datas.get(1) == null || StringUtils.isBlank(datas.get(1))){
                        planprojectdetailVO.setErrorMsg("类别及项目名称不能为空");
                        errorList.add(planprojectdetailVO);
                        continue;
                    }else{
                        planprojectdetailVO.setCategoryAndName(datas.get(1));
                    }

                    successList.add(planprojectdetailVO);
                }
            }
        }

        //设置tid和tpid
        List<ChangePlanprojectdetailVO> resList = setTidAndTpid(successList);

        JSONObject json = new JSONObject();
        json.put("successNum", successList.size());
        json.put("successList", resList);
        json.put("errorList", errorList);
        json.put("errorNum", errorList.size());

        ChangePlanprojectVO resVo = new ChangePlanprojectVO();
        json.put("total", getTotalAmount(resList, resVo));

        return CommonResponse.success(json);
    }

    private ChangePlanprojectVO getTotalAmount(List<ChangePlanprojectdetailVO> vos, ChangePlanprojectVO resVo) {
        BigDecimal costEstimateTotal = new BigDecimal(0);
        BigDecimal reportAmountTotal = new BigDecimal(0);
        BigDecimal targetAmountTotal = new BigDecimal(0);
        BigDecimal completeAmountTotal = new BigDecimal(0);
        BigDecimal estimateProfitsTotal = new BigDecimal(0);

        for (ChangePlanprojectdetailVO vo : vos) {
            if (vo.getCostEstimate() != null) {
                costEstimateTotal = costEstimateTotal.add(vo.getCostEstimate());
            }
            if (vo.getReportAmount() != null) {
                reportAmountTotal = reportAmountTotal.add(vo.getReportAmount());
            }

            if (vo.getTargetAmount() != null) {
                targetAmountTotal = targetAmountTotal.add(vo.getTargetAmount());
            }
            if (vo.getCompleteAmount() != null) {
                completeAmountTotal = completeAmountTotal.add(vo.getCompleteAmount());
            }

            if (vo.getEstimateProfits() != null) {
                estimateProfitsTotal = estimateProfitsTotal.add(vo.getEstimateProfits());
            }
        }

        resVo.setCostEstimateTotal(costEstimateTotal);
        resVo.setReportAmountTotal(reportAmountTotal);
        resVo.setTargetAmountTotal(targetAmountTotal);
        resVo.setCompleteAmountTotal(completeAmountTotal);
        resVo.setEstimateProfitsTotal(estimateProfitsTotal);
        return resVo;
    }

    /**
     * 设置tid和tpid
     * @param originalList
     * @return
     */
    private List<ChangePlanprojectdetailVO> setTidAndTpid(List<ChangePlanprojectdetailVO> originalList) {
        List<ChangePlanprojectdetailVO> treeList = new ArrayList<>();
        String currentTime = String.valueOf(System.currentTimeMillis());
        if (originalList != null && originalList.size() > 0) {
            for (int i = 0; i < originalList.size(); i++) {
                ChangePlanprojectdetailVO planprojectdetailVO = originalList.get(i);
                String sort = planprojectdetailVO.getSort();
                String[] arr = sort.split("\\.");

                if (arr.length == 1) {
                    planprojectdetailVO.setTid(currentTime + "_" + sort);
                    planprojectdetailVO.setTpid("");
                    treeList.add(planprojectdetailVO);
                } else if (arr.length > 1) {
                    planprojectdetailVO.setTid(currentTime + "_" + sort);
                    planprojectdetailVO.setTpid(currentTime + "_" + sort.substring(0, sort.length() - 2));
                    treeList.add(planprojectdetailVO);
                }
            }
        }
        return createTreeData(treeList);
    }

    /**
     * 构建树
     * @param list
     * @return
     */
    public static List<ChangePlanprojectdetailVO> createTreeData(List<ChangePlanprojectdetailVO> list) {
        List<ChangePlanprojectdetailVO> resp = new ArrayList<>();
        List<String> rootItems = new ArrayList<String>();

        //循环list，放入listMap重
        Map<String, ChangePlanprojectdetailVO> listMap = new HashMap<>();
        for (ChangePlanprojectdetailVO item : list) {
            listMap.put(item.getTid().toString(), item);
        }

        for (int i = 0; i < list.size(); i++) {
            ChangePlanprojectdetailVO item = list.get(i);
            String parentId = (item.getTpid() != null) ? item.getTpid().toString() : "";
            ChangePlanprojectdetailVO parent = listMap.get(parentId);
            if (parent != null) {
                List<ChangePlanprojectdetailVO> child = (List<ChangePlanprojectdetailVO>) parent.getChildren();
                if (child != null) {
                    child.add(item);
                } else {
                    List<ChangePlanprojectdetailVO> children = new ArrayList<ChangePlanprojectdetailVO>();
                    children.add(item);
                    parent.setChildren(children);
                }
            } else {
                rootItems.add(item.getTid());
            }
        }

        for (String rootId : rootItems) {
            resp.add(listMap.get(rootId));
        }

        return resp;
    }

    /**
     * excel导出数据库数据
     *
     * @param response
     * @return
     */
    @RequestMapping(value = "/excelExportPlanprojectFromDatabase", method = RequestMethod.POST)
    @ResponseBody
    public void excelExportbusinessplanningeffectFromDatabase(@RequestBody ChangePlanprojectVO planprojectVO, HttpServletResponse response) {
        QueryWrapper<ChangePlanprojectdetailEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("mid", planprojectVO.getId());
        queryWrapper.eq("dr", 0);
        List<ChangePlanprojectdetailEntity> list = planprojectdetailService.list(queryWrapper);
        List<ChangePlanprojectdetailVO> resList = BeanMapper.mapList(list, ChangePlanprojectdetailVO.class);

        //构建树
        resList = createTreeData(resList);

        List<ChangePlanprojectdetailVO> records = new ArrayList<>();
        for(ChangePlanprojectdetailVO vo : resList){
            records.add(vo);
            if(vo.getChildren()!=null&&vo.getChildren().size()>0){
                this.splitList(vo.getChildren(),records);
            }
        }

        Map<String, Object> beans = new HashMap<String, Object>();
        beans.put("records", handleSpecialField(records));
        ExcelExport.getInstance().export("planproject-export.xlsx", beans, response);
    }

    /**
     * 导出遍历加上所有子表
     * @param list
     * @param records
     */
    private void splitList(List<ChangePlanprojectdetailVO> list,List<ChangePlanprojectdetailVO> records) {
        for(ChangePlanprojectdetailVO vo : list){
            records.add(vo);
            if(vo.getChildren()!=null&&vo.getChildren().size()>0){
                this.splitList(vo.getChildren(),records);
            }
        }
    }

    /**
     * 处理特殊字段 如：日期
     *
     * @param resList
     * @return
     */
    private List<ChangePlanprojectdetailVO> handleSpecialField(List<ChangePlanprojectdetailVO> resList) {
        //处理日期字段
        for (int i = 0; i < resList.size(); i++) {
            ChangePlanprojectdetailVO planprojectdetailVO = resList.get(i);
            if (planprojectdetailVO.getCompleteTime() != null) {
                planprojectdetailVO.setCompleteTimeShow(DateUtils.dateSimple(planprojectdetailVO.getCompleteTime()));
            }
            if (planprojectdetailVO.getInitiateDate() != null) {
                planprojectdetailVO.setInitiateDateShow(DateUtils.dateSimple(planprojectdetailVO.getInitiateDate()));
            }
            if (planprojectdetailVO.getProposedCompleteTime() != null) {
                planprojectdetailVO.setProposedCompleteTimeShow(DateUtils.dateSimple(planprojectdetailVO.getProposedCompleteTime()));
            }
            if (planprojectdetailVO.getFinalCompleteTime() != null) {
                planprojectdetailVO.setFinalCompleteTimeShow(DateUtils.dateSimple(planprojectdetailVO.getFinalCompleteTime()));
            }
        }

        return resList;
    }

    /**
     * excel导出页面数据
     *
     * @param response
     * @return
     */
    @RequestMapping(value = "/excelExportPlanprojectFromPage", method = RequestMethod.POST)
    @ResponseBody
    public void excelExportbusinessplanningeffectFromPage(@RequestBody List<ChangePlanprojectdetailVO> list, HttpServletResponse response) {
        Map<String, Object> beans = new HashMap<String, Object>();
        List<ChangePlanprojectdetailVO> records = new ArrayList<>();

        for(ChangePlanprojectdetailVO vo : list){
            records.add(vo);
            if(vo.getChildren()!=null&&vo.getChildren().size()>0){
                this.splitList(vo.getChildren(),records);
            }
        }

        beans.put("records", handleSpecialField(records));
        ExcelExport.getInstance().export("planproject-export.xlsx", beans, response);
    }
}

==> ejc-businessplan-impl-3.0.0-20240906.083534-186-sources.jar: com/ejianc/business/change/excel/ExcelChangePlanprojectController.java <==
package com.ejianc.business.change.excel;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ejianc.business.change.bean.ChangePlanprojectdetailEntity;
import com.ejianc.business.change.service.IChangePlanprojectdetailService;
import com.ejianc.business.change.vo.ChangePlanprojectVO;
import com.ejianc.business.change.vo.ChangePlanprojectdetailVO;
import com.ejianc.business.utils.DateUtils;
import com.ejianc.framework.core.exception.BusinessException;
import com.ejianc.framework.core.kit.mapper.BeanMapper;
import com.ejianc.framework.core.response.CommonResponse;
import com.ejianc.framework.core.util.ExcelExport;
import com.ejianc.framework.core.util.ExcelReader;
import com.ejianc.framework.core.util.FileUtils;
import com.ejianc.framework.core.util.ImportTemplate;
import com.ejianc.support.idworker.util.IdWorker;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.Serializable;
import java.math.BigDecimal;
import java.text.ParseException;
import java.util.*;

/**
 * 项目商务策划立项导入导出
 */
@Controller
@RequestMapping("changePlanprojectExport")
public class ExcelChangePlanprojectController implements Serializable {
    private static final long serialVersionUID = 1L;

    private Logger logger = LoggerFactory.getLogger(this.getClass());
    @Autowired
    private IChangePlanprojectdetailService planprojectdetailService;

    /**
     * 导入模板下载
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/downloadPlanproject")
    @ResponseBody
    public void downloadTotalPrice(HttpServletRequest request, HttpServletResponse response) {
        ImportTemplate.initialize(response);
        ImportTemplate.templetdownload(request, "planproject-import.xlsx", "项目商务策划立项清单导入模板");
    }

    /**
     * excel导入
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/excelImportPlanproject", method = RequestMethod.POST)
    @ResponseBody
    public CommonResponse<Object> excelImportTotalPric(HttpServletRequest request, HttpServletResponse response) throws ParseException {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        List<ChangePlanprojectdetailVO> successList = new ArrayList<>();
        List<ChangePlanprojectdetailVO> errorList = new ArrayList<>();
        boolean isFailed = false;
        MultipartFile mf = null;
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            mf = entity.getValue();
            String originalFileName = mf.getOriginalFilename();
            String extName = null;
            originalFileName = originalFileName.replaceAll("\\/|\\/|\\||:|\\?|\\*|\"|<|>|\\p{Cntrl}", "_");
            originalFileName.replaceAll("00.", "");
            extName = FileUtils.getFileExt(originalFileName, false);
            if (!"xls".equals(extName) && !"xlsx".equals(extName)) {
                isFailed = true;
                break;
            }
        }
        if (isFailed) {
            return CommonResponse.error("文件格式不合法");
        } else {
            List<List<String>> result = ExcelReader.readExcel(mf);
            if (result != null && result.size() > 0) {
                for (int i = 1; i < result.size(); i++) {
                    List<String> datas = result.get(i);
                    ChangePlanprojectdetailVO planprojectdetailVO = new ChangePlanprojectdetailVO();
                    planprojectdetailVO.setId(IdWorker.getId());//id
                    String sort = datas.get(0);
                    planprojectdetailVO.setPlanCategory(StringUtils.isNotBlank(datas.get(2)) ? datas.get(2) : null); // 策划类别
                    planprojectdetailVO.setPlanType(StringUtils.isNotBlank(datas.get(3)) ? datas.get(3) : null); // 策划类型
                    planprojectdetailVO.setPlanContent(StringUtils.isNotBlank(datas.get(4)) ? datas.get(4) : null); //策划内容
                    planprojectdetailVO.setPlanStatus(StringUtils.isNotBlank(datas.get(5)) ? datas.get(5): null); // 策划状态
                    planprojectdetailVO.setTargetAmount(StringUtils.isNotBlank(datas.get(6)) ? new BigDecimal(datas.get(6)) : null); // 策划目标（万元）
                    planprojectdetailVO.setTakenMeasures(StringUtils.isNotBlank(datas.get(7)) ? datas.get(7) : null); // 拟采取的措施
                    planprojectdetailVO.setPersonLiable(StringUtils.isNotBlank(datas.get(8)) ? datas.get(8) : null); // 责任人
                    String arr = datas.get(9);

                    if (DateUtils.grepDate(arr)) {
                        planprojectdetailVO.setProposedImplementationTime(StringUtils.isNotBlank(datas.get(9)) ? DateUtils.Date(datas.get(9)) : null); // 拟实施时间
                    } else {
                        planprojectdetailVO.setErrorMsg("日期格式不正确");
                        errorList.add(planprojectdetailVO);
                        continue;
                    }
                    if(sort==null || sort == ""){
                        planprojectdetailVO.setErrorMsg("序号不能为空");
                        errorList.add(planprojectdetailVO);
                        continue;
                    }else{
                        planprojectdetailVO.setSort(datas.get(0));
                    }
                    if(datas.get(1) == null || StringUtils.isBlank(datas.get(1))){
                        planprojectdetailVO.setErrorMsg("策划专项名称不能为空");
                        errorList.add(planprojectdetailVO);
                        continue;
                    }else{
                        planprojectdetailVO.setCategoryAndName(datas.get(1));
                    }

                    successList.add(planprojectdetailVO);
                }
            }
        }

        //设置tid和tpid
        List<ChangePlanprojectdetailVO> resList = setTidAndTpid(successList);

        JSONObject json = new JSONObject();
        json.put("successNum", successList.size());
        json.put("successList", resList);
        json.put("errorList", errorList);
        json.put("errorNum", errorList.size());

        ChangePlanprojectVO resVo = new ChangePlanprojectVO();
        json.put("total", getTotalAmount(resList, resVo));

        return CommonResponse.success(json);
    }

    private ChangePlanprojectVO getTotalAmount(List<ChangePlanprojectdetailVO> vos, ChangePlanprojectVO resVo) {
        BigDecimal costEstimateTotal = new BigDecimal(0);
        BigDecimal reportAmountTotal = new BigDecimal(0);
        BigDecimal targetAmountTotal = new BigDecimal(0);
        BigDecimal completeAmountTotal = new BigDecimal(0);
        BigDecimal estimateProfitsTotal = new BigDecimal(0);

        for (ChangePlanprojectdetailVO vo : vos) {
            if (vo.getCostEstimate() != null) {
                costEstimateTotal = costEstimateTotal.add(vo.getCostEstimate());
            }
            if (vo.getReportAmount() != null) {
                reportAmountTotal = reportAmountTotal.add(vo.getReportAmount());
            }

            if (vo.getTargetAmount() != null) {
                targetAmountTotal = targetAmountTotal.add(vo.getTargetAmount());
            }
            if (vo.getCompleteAmount() != null) {
                completeAmountTotal = completeAmountTotal.add(vo.getCompleteAmount());
            }

            if (vo.getEstimateProfits() != null) {
                estimateProfitsTotal = estimateProfitsTotal.add(vo.getEstimateProfits());
            }
        }

        resVo.setCostEstimateTotal(costEstimateTotal);
        resVo.setReportAmountTotal(reportAmountTotal);
        resVo.setTargetAmountTotal(targetAmountTotal);
        resVo.setCompleteAmountTotal(completeAmountTotal);
        resVo.setEstimateProfitsTotal(estimateProfitsTotal);
        return resVo;
    }

    /**
     * 设置tid和tpid
     * @param originalList
     * @return
     */
    private List<ChangePlanprojectdetailVO> setTidAndTpid(List<ChangePlanprojectdetailVO> originalList) {
        List<ChangePlanprojectdetailVO> treeList = new ArrayList<>();
        String currentTime = String.valueOf(System.currentTimeMillis());
        if (originalList != null && originalList.size() > 0) {
            for (int i = 0; i < originalList.size(); i++) {
                ChangePlanprojectdetailVO planprojectdetailVO = originalList.get(i);
                String sort = planprojectdetailVO.getSort();
                String[] arr = sort.split("\\.");

                if (arr.length == 1) {
                    planprojectdetailVO.setTid(currentTime + "_" + sort);
                    planprojectdetailVO.setTpid("");
                    treeList.add(planprojectdetailVO);
                } else if (arr.length > 1) {
                    planprojectdetailVO.setTid(currentTime + "_" + sort);
                    planprojectdetailVO.setTpid(currentTime + "_" + sort.substring(0, sort.length() - 2));
                    treeList.add(planprojectdetailVO);
                }
            }
        }
        return createTreeData(treeList);
    }

    /**
     * 构建树
     * @param list
     * @return
     */
    public static List<ChangePlanprojectdetailVO> createTreeData(List<ChangePlanprojectdetailVO> list) {
        List<ChangePlanprojectdetailVO> resp = new ArrayList<>();
        List<String> rootItems = new ArrayList<String>();

        //循环list，放入listMap重
        Map<String, ChangePlanprojectdetailVO> listMap = new HashMap<>();
        for (ChangePlanprojectdetailVO item : list) {
            listMap.put(item.getTid().toString(), item);
        }

        for (int i = 0; i < list.size(); i++) {
            ChangePlanprojectdetailVO item = list.get(i);
            String parentId = (item.getTpid() != null) ? item.getTpid().toString() : "";
            ChangePlanprojectdetailVO parent = listMap.get(parentId);
            if (parent != null) {
                List<ChangePlanprojectdetailVO> child = (List<ChangePlanprojectdetailVO>) parent.getChildren();
                if (child != null) {
                    child.add(item);
                } else {
                    List<ChangePlanprojectdetailVO> children = new ArrayList<ChangePlanprojectdetailVO>();
                    children.add(item);
                    parent.setChildren(children);
                }
            } else {
                rootItems.add(item.getTid());
            }
        }

        for (String rootId : rootItems) {
            resp.add(listMap.get(rootId));
        }

        return resp;
    }

    /**
     * excel导出数据库数据
     *
     * @param response
     * @return
     */
    @RequestMapping(value = "/excelExportPlanprojectFromDatabase", method = RequestMethod.POST)
    @ResponseBody
    public void excelExportbusinessplanningeffectFromDatabase(@RequestBody ChangePlanprojectVO planprojectVO, HttpServletResponse response) {
        QueryWrapper<ChangePlanprojectdetailEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("mid", planprojectVO.getId());
        queryWrapper.eq("dr", 0);
        List<ChangePlanprojectdetailEntity> list = planprojectdetailService.list(queryWrapper);
        List<ChangePlanprojectdetailVO> resList = BeanMapper.mapList(list, ChangePlanprojectdetailVO.class);

        //构建树
        resList = createTreeData(resList);

        List<ChangePlanprojectdetailVO> records = new ArrayList<>();
        for(ChangePlanprojectdetailVO vo : resList){
            records.add(vo);
            if(vo.getChildren()!=null&&vo.getChildren().size()>0){
                this.splitList(vo.getChildren(),records);
            }
        }

        Map<String, Object> beans = new HashMap<String, Object>();
        beans.put("records", handleSpecialField(records));
        ExcelExport.getInstance().export("planproject-export.xlsx", beans, response);
    }

    /**
     * 导出遍历加上所有子表
     * @param list
     * @param records
     */
    private void splitList(List<ChangePlanprojectdetailVO> list,List<ChangePlanprojectdetailVO> records) {
        for(ChangePlanprojectdetailVO vo : list){
            records.add(vo);
            if(vo.getChildren()!=null&&vo.getChildren().size()>0){
                this.splitList(vo.getChildren(),records);
            }
        }
    }

    /**
     * 处理特殊字段 如：日期
     *
     * @param resList
     * @return
     */
    private List<ChangePlanprojectdetailVO> handleSpecialField(List<ChangePlanprojectdetailVO> resList) {
        //处理日期字段
        for (int i = 0; i < resList.size(); i++) {
            ChangePlanprojectdetailVO planprojectdetailVO = resList.get(i);
            if (planprojectdetailVO.getCompleteTime() != null) {
                planprojectdetailVO.setCompleteTimeShow(DateUtils.dateSimple(planprojectdetailVO.getCompleteTime()));
            }
            if (planprojectdetailVO.getInitiateDate() != null) {
                planprojectdetailVO.setInitiateDateShow(DateUtils.dateSimple(planprojectdetailVO.getInitiateDate()));
            }
            if (planprojectdetailVO.getProposedCompleteTime() != null) {
                planprojectdetailVO.setProposedCompleteTimeShow(DateUtils.dateSimple(planprojectdetailVO.getProposedCompleteTime()));
            }
            if (planprojectdetailVO.getFinalCompleteTime() != null) {
                planprojectdetailVO.setFinalCompleteTimeShow(DateUtils.dateSimple(planprojectdetailVO.getFinalCompleteTime()));
            }
        }

        return resList;
    }

    /**
     * excel导出页面数据
     *
     * @param response
     * @return
     */
    @RequestMapping(value = "/excelExportPlanprojectFromPage", method = RequestMethod.POST)
    @ResponseBody
    public void excelExportbusinessplanningeffectFromPage(@RequestBody List<ChangePlanprojectdetailVO> list, HttpServletResponse response) {
        Map<String, Object> beans = new HashMap<String, Object>();
        List<ChangePlanprojectdetailVO> records = new ArrayList<>();

        for(ChangePlanprojectdetailVO vo : list){
            records.add(vo);
            if(vo.getChildren()!=null&&vo.getChildren().size()>0){
                this.splitList(vo.getChildren(),records);
            }
        }

        beans.put("records", handleSpecialField(records));
        ExcelExport.getInstance().export("planproject-export.xlsx", beans, response);
    }
}
